"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hashSmallFile = exports.hashFile = void 0;
const tslib_1 = require("tslib");
const crypto = (0, tslib_1.__importStar)(require("crypto"));
const fs = (0, tslib_1.__importStar)(require("fs-extra"));
const os = (0, tslib_1.__importStar)(require("os"));
const path = (0, tslib_1.__importStar)(require("path"));
const semaphore_async_await_1 = (0, tslib_1.__importDefault)(require("semaphore-async-await"));
const worker_threads_1 = require("worker_threads");
const jobs = os.cpus().length;
const semaphore = new semaphore_async_await_1.default(jobs);
class ThreadFileHasher {
    constructor(fileName, thread, fnResolve, fnReject) {
        this.fnResolve = fnResolve;
        this.fnReject = fnReject;
        this.thread = thread;
        thread.on("message", (message) => {
            if (!message.directive) {
                this.reject(new Error("IPC Error: " + message));
                return;
            }
            switch (message.directive) {
                case "ready":
                    thread.postMessage({ directive: "compute", path: fileName });
                    return;
                case "return":
                    this.resolve(message.result);
                    return;
                default:
                    this.reject(new Error("IPC Error: " + message));
                    return;
            }
        });
        thread.on("error", (e) => this.reject(e));
    }
    resolve(x) {
        if (this.thread) {
            this.thread.unref();
            this.thread = null;
            setImmediate(() => this.fnResolve(x));
        }
    }
    reject(x) {
        if (this.thread) {
            this.thread.unref();
            this.thread = null;
            setImmediate(() => this.fnReject(x));
        }
    }
}
function hashFileChildProcessImpl(fileName) {
    return new Promise((resolve, reject) => {
        new ThreadFileHasher(fileName, new worker_threads_1.Worker(path.join(__dirname, "thread.js"), { stdout: true }), resolve, reject);
    });
}
async function hashFile(path) {
    await semaphore.acquire();
    try {
        return await hashFileChildProcessImpl(path);
    }
    finally {
        semaphore.release();
    }
}
exports.hashFile = hashFile;
async function hashSmallFile(path) {
    return new Promise((resolve, reject) => {
        let sum = crypto.createHash("sha1");
        let fileStream = fs.createReadStream(path);
        fileStream.on("error", function (err) {
            return reject(err);
        });
        fileStream.on("data", function (chunk) {
            try {
                sum.update(chunk);
            }
            catch (ex) {
                return reject(ex);
            }
        });
        fileStream.on("end", function () {
            return resolve(sum.digest("hex"));
        });
    });
}
exports.hashSmallFile = hashSmallFile;
