"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const crypto = (0, tslib_1.__importStar)(require("crypto"));
const fs = (0, tslib_1.__importStar)(require("fs-extra"));
const worker_threads_1 = require("worker_threads");
if (!worker_threads_1.parentPort) {
    throw new Error("This script only works in worker threads.");
}
function hashFile(path) {
    return new Promise((resolve, reject) => {
        let sum = crypto.createHash("sha1");
        let fileStream = fs.createReadStream(path);
        fileStream.on("error", (err) => {
            return reject(err);
        });
        fileStream.on("data", (chunk) => {
            try {
                sum.update(chunk);
            }
            catch (ex) {
                return reject(ex);
            }
        });
        fileStream.on("end", () => {
            return resolve(sum.digest("hex"));
        });
    });
}
worker_threads_1.parentPort.on("message", (message) => {
    switch (message.directive) {
        case "compute":
            if (!message.path) {
                worker_threads_1.parentPort.unref();
                throw new Error("Path not found.");
            }
            hashFile(message.path)
                .then((result) => {
                worker_threads_1.parentPort.postMessage({ directive: "return", result });
                worker_threads_1.parentPort.unref();
            })
                .catch((e) => {
                worker_threads_1.parentPort.unref();
                throw e;
            });
            break;
        default:
            worker_threads_1.parentPort.unref();
            throw new Error("Message directive not recognized.");
    }
});
setTimeout(() => worker_threads_1.parentPort.postMessage({ directive: "ready" }), 0);
