"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const cliCursor = (0, tslib_1.__importStar)(require("cli-cursor"));
const readline = (0, tslib_1.__importStar)(require("readline"));
const isLegacyConsole = process.platform === "win32" &&
    !(process.env.TERM_PROGRAM === "vscode" || !!process.env.WT_SESSION);
const ansiRegex = /(\u009B|\u001B\[)[0-?]*[ -\/]*[@-~]/g;
const spinnerInterval = 100;
const spinnerFrames = [
    "⢀⠀",
    "⡀⠀",
    "⠄⠀",
    "⢂⠀",
    "⡂⠀",
    "⠅⠀",
    "⢃⠀",
    "⡃⠀",
    "⠍⠀",
    "⢋⠀",
    "⡋⠀",
    "⠍⠁",
    "⢋⠁",
    "⡋⠁",
    "⠍⠉",
    "⠋⠉",
    "⠋⠉",
    "⠉⠙",
    "⠉⠙",
    "⠉⠩",
    "⠈⢙",
    "⠈⡙",
    "⢈⠩",
    "⡀⢙",
    "⠄⡙",
    "⢂⠩",
    "⡂⢘",
    "⠅⡘",
    "⢃⠨",
    "⡃⢐",
    "⠍⡐",
    "⢋⠠",
    "⡋⢀",
    "⠍⡁",
    "⢋⠁",
    "⡋⠁",
    "⠍⠉",
    "⠋⠉",
    "⠋⠉",
    "⠉⠙",
    "⠉⠙",
    "⠉⠩",
    "⠈⢙",
    "⠈⡙",
    "⠈⠩",
    "⠀⢙",
    "⠀⡙",
    "⠀⠩",
    "⠀⢘",
    "⠀⡘",
    "⠀⠨",
    "⠀⢐",
    "⠀⡐",
    "⠀⠠",
    "⠀⢀",
    "⠀⡀",
];
const spinnerFramesCP437 = ["-", "\\", "|", "/"];
class Spinner {
    constructor() {
        this.textSource = null;
        this.lastTextSize = 0;
        this.frame = 0;
        this.frames = isLegacyConsole ? spinnerFramesCP437 : spinnerFrames;
        this.interval = spinnerInterval;
        this.enabled = true;
        this.currentFrameWritten = false;
        this.id = null;
        this.stream = process.stderr;
        this.outputIsTty = process.stdout.isTTY && process.stderr.isTTY && !(process.env.TERM === "dumb");
    }
    clear() {
        if (!this.enabled || !this.outputIsTty)
            return;
        readline.clearLine(this.stream, 0);
        readline.cursorTo(this.stream, 0);
        this.lastTextSize = 0;
    }
    getText() {
        return this.textSource ? this.textSource.getText() : "";
    }
    nextFrame() {
        if (!this.enabled || !this.outputIsTty)
            return;
        readline.cursorTo(this.stream, 0);
        const text = this.getText();
        const sizeText = this.textSize(text);
        const trailing = sizeText < this.lastTextSize ? " ".repeat(this.lastTextSize - sizeText) : "";
        this.stream.write(chalk_1.default.cyan(this.frames[this.frame]) + " " + text + trailing);
        this.lastTextSize = sizeText;
        if (!this.currentFrameWritten)
            this.frame = (this.frame + 1) % this.frames.length;
        this.currentFrameWritten = true;
    }
    timerCallback() {
        this.currentFrameWritten = false;
        this.nextFrame();
    }
    start() {
        this.enabled = true;
        if (this.id !== null)
            clearInterval(this.id);
        this.nextFrame();
        this.id = setInterval(() => this.timerCallback(), this.interval);
        cliCursor.hide(this.stream);
    }
    pause() {
        if (this.id !== null)
            clearInterval(this.id);
        this.clear();
        cliCursor.show(this.stream);
        this.enabled = false;
    }
    stop() {
        this.pause();
        this.stream.write(chalk_1.default.cyan(this.getText()));
        this.enabled = false;
    }
    textSize(t) {
        return t.replace(ansiRegex, "").length;
    }
}
exports.default = Spinner;
